@@ -1,4 +1,5 @@
 import datetime
+import json
 import logging
 import ssl
 import sys
@@ -6,21 +7,21 @@ import unittest
 import warnings
 
 import mock
-from nose.plugins.skip import SkipTest
+import pytest
 
 from dummyserver.testcase import (
     HTTPSDummyServerTestCase, IPV6HTTPSDummyServerTestCase
 )
 from dummyserver.server import (DEFAULT_CA, DEFAULT_CA_BAD, DEFAULT_CERTS,
+                                DEFAULT_CLIENT_CERTS,
+                                DEFAULT_CLIENT_NO_INTERMEDIATE_CERTS,
                                 NO_SAN_CERTS, NO_SAN_CA, DEFAULT_CA_DIR,
                                 IPV6_ADDR_CERTS, IPV6_ADDR_CA, HAS_IPV6,
                                 IP_SAN_CERTS)
 
 from test import (
-    onlyPy26OrOlder,
     onlyPy279OrNewer,
     notSecureTransport,
-    onlyPy27OrNewerOrNonWindows,
     requires_network,
     TARPIT_HOST,
 )
@@ -37,6 +38,7 @@ from urllib3.exceptions import (
     SystemTimeWarning,
     InsecurePlatformWarning,
     MaxRetryError,
+    ProtocolError,
 )
 from urllib3.packages import six
 from urllib3.util.timeout import Timeout
@@ -62,11 +64,52 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         r = self._pool.request('GET', '/')
         self.assertEqual(r.status, 200, r.data)
 
+    def test_dotted_fqdn(self):
+        pool = HTTPSConnectionPool(self.host + '.', self.port)
+        r = pool.request('GET', '/')
+        self.assertEqual(r.status, 200, r.data)
+
     def test_set_ssl_version_to_tlsv1(self):
         self._pool.ssl_version = ssl.PROTOCOL_TLSv1
         r = self._pool.request('GET', '/')
         self.assertEqual(r.status, 200, r.data)
 
+    def test_client_intermediate(self):
+        client_cert, client_key = (
+            DEFAULT_CLIENT_CERTS['certfile'],
+            DEFAULT_CLIENT_CERTS['keyfile'],
+        )
+        https_pool = HTTPSConnectionPool(self.host, self.port,
+                                         key_file=client_key,
+                                         cert_file=client_cert)
+        r = https_pool.request('GET', '/certificate')
+        subject = json.loads(r.data.decode('utf-8'))
+        assert subject['organizationalUnitName'].startswith(
+            'Testing server cert')
+
+    def test_client_no_intermediate(self):
+        client_cert, client_key = (
+            DEFAULT_CLIENT_NO_INTERMEDIATE_CERTS['certfile'],
+            DEFAULT_CLIENT_NO_INTERMEDIATE_CERTS['keyfile']
+        )
+        https_pool = HTTPSConnectionPool(self.host, self.port,
+                                         cert_file=client_cert,
+                                         key_file=client_key)
+        try:
+            https_pool.request('GET', '/certificate', retries=False)
+        except SSLError as e:
+            if not ('alert unknown ca' in str(e) or
+                    'invalid certificate chain' in str(e) or
+                    'unknown Cert Authority' in str(e) or
+                    # https://github.com/urllib3/urllib3/issues/1422
+                    'connection closed via error' in str(e) or
+                    'WSAECONNRESET' in str(e)):
+                raise
+        except ProtocolError as e:
+            # https://github.com/urllib3/urllib3/issues/1422
+            if not ('An existing connection was forcibly closed by the remote host' in str(e)):
+                raise
+
     def test_verified(self):
         https_pool = HTTPSConnectionPool(self.host, self.port,
                                          cert_reqs='CERT_REQUIRED',
@@ -82,7 +125,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
             # Modern versions of Python, or systems using PyOpenSSL, don't
             # emit warnings.
-            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
+            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL \
+                    or util.IS_SECURETRANSPORT:
                 self.assertFalse(warn.called, warn.call_args_list)
             else:
                 self.assertTrue(warn.called)
@@ -109,7 +153,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
             # Modern versions of Python, or systems using PyOpenSSL, don't
             # emit warnings.
-            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
+            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL \
+                    or util.IS_SECURETRANSPORT:
                 self.assertFalse(warn.called, warn.call_args_list)
             else:
                 self.assertTrue(warn.called)
@@ -136,7 +181,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
             # Modern versions of Python, or systems using PyOpenSSL, don't
             # emit warnings.
-            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
+            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL \
+                    or util.IS_SECURETRANSPORT:
                 self.assertFalse(warn.called, warn.call_args_list)
             else:
                 self.assertTrue(warn.called)
@@ -148,7 +194,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                 self.assertEqual(error, InsecurePlatformWarning)
 
     @onlyPy279OrNewer
-    @notSecureTransport
+    @notSecureTransport  # SecureTransport does not support cert directories
     def test_ca_dir_verified(self):
         https_pool = HTTPSConnectionPool(self.host, self.port,
                                          cert_reqs='CERT_REQUIRED',
@@ -190,9 +236,9 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             self.fail("Didn't raise SSL error with bad CA certs")
         except MaxRetryError as e:
             self.assertIsInstance(e.reason, SSLError)
-            self.assertTrue('certificate verify failed' in str(e.reason),
-                            "Expected 'certificate verify failed',"
-                            "instead got: %r" % e.reason)
+            self.assertIn('certificate verify failed', str(e.reason),
+                          "Expected 'certificate verify failed',"
+                          "instead got: %r" % e.reason)
 
     def test_verified_without_ca_certs(self):
         # default is cert_reqs=None which is ssl.CERT_NONE
@@ -240,13 +286,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             # the unverified warning. Older systems may also emit other
             # warnings, which we want to ignore here.
             calls = warn.call_args_list
-            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
-                category = calls[0][0][1]
-            elif util.HAS_SNI:
-                category = calls[1][0][1]
-            else:
-                category = calls[2][0][1]
-            self.assertEqual(category, InsecureRequestWarning)
+            self.assertIn(InsecureRequestWarning, [x[0][1] for x in calls])
 
     def test_ssl_unverified_with_ca_certs(self):
         pool = HTTPSConnectionPool(self.host, self.port,
@@ -263,7 +303,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             # the unverified warning. Older systems may also emit other
             # warnings, which we want to ignore here.
             calls = warn.call_args_list
-            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
+            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL \
+                    or util.IS_SECURETRANSPORT:
                 category = calls[0][0][1]
             elif util.HAS_SNI:
                 category = calls[1][0][1]
@@ -289,6 +330,23 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool.assert_hostname = 'localhost'
         https_pool.request('GET', '/')
 
+    def test_server_hostname(self):
+        https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
+                                         cert_reqs='CERT_REQUIRED',
+                                         ca_certs=DEFAULT_CA,
+                                         server_hostname='localhost')
+        self.addCleanup(https_pool.close)
+
+        conn = https_pool._new_conn()
+        conn.request('GET', '/')
+
+        # Assert the wrapping socket is using the passed-through SNI name.
+        # pyopenssl doesn't let you pull the server_hostname back off the
+        # socket, so only add this assertion if the attribute is there (i.e.
+        # the python ssl module).
+        if hasattr(conn.sock, 'server_hostname'):
+            self.assertEqual(conn.sock.server_hostname, "localhost")
+
     def test_assert_fingerprint_md5(self):
         https_pool = HTTPSConnectionPool('localhost', self.port,
                                          cert_reqs='CERT_REQUIRED',
@@ -422,26 +480,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         self.addCleanup(https_pool.close)
         conn = https_pool._new_conn()
         self.addCleanup(conn.close)
-        try:
-            conn.set_tunnel(self.host, self.port)
-        except AttributeError:  # python 2.6
-            conn._set_tunnel(self.host, self.port)
+        conn.set_tunnel(self.host, self.port)
         conn._tunnel = mock.Mock()
         https_pool._make_request(conn, 'GET', '/')
         conn._tunnel.assert_called_once_with()
 
-    @onlyPy26OrOlder
-    def test_tunnel_old_python(self):
-        """HTTPSConnection can still make connections if _tunnel_host isn't set
-
-        The _tunnel_host attribute was added in 2.6.3 - because our test runners
-        generally use the latest Python 2.6, we simulate the old version by
-        deleting the attribute from the HTTPSConnection.
-        """
-        conn = self._pool._new_conn()
-        del conn._tunnel_host
-        self._pool._make_request(conn, 'GET', '/')
-
     @requires_network
     def test_enhanced_timeout(self):
         def new_pool(timeout, cert_reqs='CERT_REQUIRED'):
@@ -481,6 +524,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
         https_pool._make_request(conn, 'GET', '/')
 
+    @onlyPy279OrNewer
     def test_ssl_correct_system_time(self):
         self._pool.cert_reqs = 'CERT_REQUIRED'
         self._pool.ca_certs = DEFAULT_CA
@@ -488,6 +532,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         w = self._request_without_resource_warnings('GET', '/')
         self.assertEqual([], w)
 
+    @onlyPy279OrNewer
     def test_ssl_wrong_system_time(self):
         self._pool.cert_reqs = 'CERT_REQUIRED'
         self._pool.ca_certs = DEFAULT_CA
@@ -500,7 +545,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             warning = w[0]
 
             self.assertEqual(SystemTimeWarning, warning.category)
-            self.assertTrue(str(RECENT_DATE) in warning.message.args[0])
+            self.assertIn(str(RECENT_DATE), warning.message.args[0])
 
     def _request_without_resource_warnings(self, method, url):
         with warnings.catch_warnings(record=True) as w:
@@ -518,11 +563,7 @@ class TestHTTPS_TLSv1(HTTPSDummyServerTestCase):
         self._pool = HTTPSConnectionPool(self.host, self.port)
         self.addCleanup(self._pool.close)
 
-    @onlyPy27OrNewerOrNonWindows
     def test_discards_connection_on_sslerror(self):
-        # This test is skipped on Windows for Python 2.6 because we suspect there
-        # is an issue with the OpenSSL for Python 2.6 on Windows.
-
         self._pool.cert_reqs = 'CERT_REQUIRED'
         with self.assertRaises(MaxRetryError) as cm:
             self._pool.request('GET', '/', retries=0)
@@ -560,7 +601,7 @@ class TestHTTPS_IPSAN(HTTPSDummyServerTestCase):
         try:
             import ipaddress  # noqa: F401
         except ImportError:
-            raise SkipTest("Only runs on systems with an ipaddress module")
+            pytest.skip("Only runs on systems with an ipaddress module")
 
         https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
                                          cert_reqs='CERT_REQUIRED',
@@ -573,10 +614,9 @@ class TestHTTPS_IPSAN(HTTPSDummyServerTestCase):
 class TestHTTPS_IPv6Addr(IPV6HTTPSDummyServerTestCase):
     certs = IPV6_ADDR_CERTS
 
+    @pytest.mark.skipif(not HAS_IPV6, reason='Only runs on IPv6 systems')
     def test_strip_square_brackets_before_validating(self):
         """Test that the fix for #760 works."""
-        if not HAS_IPV6:
-            raise SkipTest("Only runs on IPv6 systems")
         https_pool = HTTPSConnectionPool('[::1]', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=IPV6_ADDR_CA)
